//----------------------------------------------------------------------------------------------------------------------
// 1. DRIVER SETUP - create driver object, midi ports and detection information
//----------------------------------------------------------------------------------------------------------------------

var midiremote_api = require('midiremote_api_v1')
var deviceDriver = midiremote_api.makeDeviceDriver('mj', 'C4-emulation', 'Martin Jirsak')

var midiInput = deviceDriver.mPorts.makeMidiInput()
var midiOutput = deviceDriver.mPorts.makeMidiOutput()

deviceDriver.makeDetectionUnit().detectPortPair(midiInput, midiOutput)
	.expectInputNameContains('Max 1')
	.expectOutputNameContains('Max 1')

var surface = deviceDriver.mSurface

//----------------------------------------------------------------------------------------------------------------------
// 2. SURFACE LAYOUT - create control elements and midi bindings
//----------------------------------------------------------------------------------------------------------------------

function makeKnobStrip(knobIndex, x, y, surfaceElements) {
	var knobStrip = {}

	knobStrip.knob = surface.makeKnob(x + 2 * knobIndex + 0.1, y, 1.8, 2.2)
	knobStrip.knob.mSurfaceValue.mMidiBinding.setInputPort(midiInput).bindToControlChange(0, knobIndex)

	return knobStrip
}

//----------------------------------------------------------------------------------------------------------------------
function makeSurfaceElements() {
	var surfaceElements = {}

	surfaceElements.numStrips = 8
	surfaceElements.knobStrips = {}

	var x = 0
	var y = 0

	for (var i = 0; i < surfaceElements.numStrips; ++i) {
		console.log(x.toString())
		surfaceElements.knobStrips[i] = makeKnobStrip(i, x, y, surfaceElements)
	}

	return surfaceElements
}

var surfaceElements = makeSurfaceElements()

//----------------------------------------------------------------------------------------------------------------------
// 3. HOST MAPPING - create mapping pages and host bindings
//----------------------------------------------------------------------------------------------------------------------

function makePageWithDefaults(name) {
	var page = deviceDriver.mMapping.makePage(name)

	return page
}

//----------------------------------------------------------------------------------------------------------------------
function makePageMixer() {
	var page = makePageWithDefaults('Mixer')

	var hostMixerBankZone = page.mHostAccess.mMixConsole.makeMixerBankZone()
		.excludeInputChannels()
		.excludeOutputChannels()

	for (var i = 0; i < 8; ++i) {
		var channelBankItem = hostMixerBankZone.makeMixerBankChannel()
		var knobValue = surfaceElements.knobStrips[i].knob.mSurfaceValue

		page.makeValueBinding(knobValue, channelBankItem.mValue.mVolume).setValueTakeOverModeScaled()
	}

	return page
}

//----------------------------------------------------------------------------------------------------------------------
// 4. Feedback to the HW controller
//----------------------------------------------------------------------------------------------------------------------
function onActivatePageDefault(context) {

	function sendFeedbackOut(button, ccNr) {
		button.mSurfaceValue.mOnProcessValueChange = function (context, newValue) {
			midiOutput.sendMidi(context, [0xb0, ccNr, Math.round(newValue * 11.99)])
			// console.log('newValue: ' + newValue)
		}
	}

	for (var i = 0; i < 8; ++i) {
		sendFeedbackOut(surfaceElements.knobStrips[i].knob, 32 + i)
	}
}

//----------------------------------------------------------------------------------------------------------------------
// Switch nanoKONTROL to expected mode
//----------------------------------------------------------------------------------------------------------------------

deviceDriver.mOnActivate = function (context) {
	console.log('Your KORG nanoKONTORL2 has been switched to the CC Mode and the LED Mode is set to External.')
	// switchDeviceToCcModeWithExternalLed(context)
}

//----------------------------------------------------------------------------------------------------------------------
// Construct Pages
//----------------------------------------------------------------------------------------------------------------------
var pageMixer = makePageMixer()

pageMixer.mOnActivate = function (context) {
	onActivatePageDefault(context)
	console.log('from script: C4 "Mixer" page activated')
}